from smtplib import SMTPException
from unittest.mock import patch
import copy

from django.conf import settings
from django.urls import path, include
from django.contrib import auth
from django.contrib.auth.models import User
from django.contrib.messages import get_messages
from django.core import mail
from django.shortcuts import reverse
from django.test import override_settings, TestCase

import tom_common.urls


class TestOpenRegistrationViews(TestCase):
    """NOTE: run tests with: python ./tom_registration/tests/run_tests.py"""
    def setUp(self):
        self.user_data = {
            'username': 'aaronrodgers',
            'first_name': 'Aaron',
            'last_name': 'Rodgers',
            'email': 'aaronrodgers@berkeley.edu',
            'password1': 'gopackgo',
            'password2': 'gopackgo',
            'profile-TOTAL_FORMS': ['1'],
            'profile-INITIAL_FORMS': ['0'],
            'profile-0-affiliation': [''],
        }

    def test_user_register(self):
        """Test that a user can register using the open registration flow."""
        response = self.client.post(reverse('registration:register'), data=self.user_data)
        user = User.objects.get(username=self.user_data['username'])

        self.assertEqual(response.status_code, 302)
        self.assertEqual(user.id, auth.get_user(self.client).id)

    @override_settings(TOM_REGISTRATION={'REGISTRATION_AUTHENTICATION_BACKEND': ''})
    def test_user_register_login_failure(self):
        """
        Test that an error message is logged correctly when a newly-registered user has an automated login failure.
        """
        del settings.REGISTRATION_AUTHENTICATION_BACKEND
        with self.assertLogs('tom_registration.registration_flows.open.views', level='ERROR') as logs:
            response = self.client.post(reverse('registration:register'), data=self.user_data)
            self.assertIn(
                'ERROR:tom_registration.registration_flows.open.views:Unable to log in newly registered user: '
                'You have multiple authentication backends configured and therefore must provide the `backend` argument'
                ' or set the `backend` attribute on the user.', logs.output)

        self.assertEqual(response.status_code, 302)
        self.assertTrue(auth.get_user(self.client).is_anonymous)


@override_settings(
    AUTHENTICATION_BACKENDS=('django.contrib.auth.backends.AllowAllUsersModelBackend',
                             'guardian.backends.ObjectPermissionBackend',
                             ),
    TOM_REGISTRATION={'REGISTRATION_AUTHENTICATION_BACKEND': 'django.contrib.auth.backends.AllowAllUsersModelBackend',
                      'REGISTRATION_REDIRECT_PATTERN': 'home',
                      'SEND_APPROVAL_EMAILS': True,
                      'REGISTRATION_STRATEGY': 'approval_required'
                      },
    MANAGERS=[('David', 'dcollom@lco.global')],
)
class TestApprovalRegistrationViews(TestCase):
    """NOTE: run tests with: python ./tom_registration/tests/run_tests.py"""

    urls = 'tom_common.urls'

    def setUp(self):
        # We need to change the URL patterns to use the approval registration flow for these tests
        # To do this we need prepend the approval URL pattern
        registration_strategy = settings.TOM_REGISTRATION['REGISTRATION_STRATEGY']

        tom_common.urls.urlpatterns.insert(0, path('', include(f'tom_registration.registration_flows.'
                                                               f'{registration_strategy}.urls',
                                                               namespace='registration')))
        self.user_data = {
            'username': 'aaronrodgers',
            'first_name': 'Aaron',
            'last_name': 'Rodgers',
            'email': 'aaronrodgers@berkeley.edu',
            'password1': 'gopackgo',
            'password2': 'gopackgo',
        }
        self.form_data = {
            'profile-TOTAL_FORMS': ['1'],
            'profile-INITIAL_FORMS': ['0'],
            'profile-0-affiliation': [''],
        }
        self.user_form_data = {
            **self.user_data,
            **self.form_data,
        }

        self.superuser = User.objects.create_superuser(username='superuser')

    def tearDown(self):
        # undo the registration URL changes made in setUp
        del tom_common.urls.urlpatterns[0]

    def test_user_register(self):
        """
        Test that a user can register using the approval registration flow, that the user is inactive, and that the user
        sees the correct error message if they attempt to log in prior to approval.
        """
        response = self.client.post(reverse('registration:register'), data=self.user_form_data)
        messages = [(m.message, m.level) for m in get_messages(response.wsgi_request)]
        user = User.objects.get(username=self.user_data['username'])

        self.assertEqual(response.status_code, 302)
        self.assertEqual(len(messages), 1)
        self.assertEqual(messages[0][0], 'Your request to register has been submitted to the administrators.')
        self.assertFalse(user.is_active)

    def test_unaproved_user_login_failure(self):
        self.client.post(reverse('registration:register'), data=self.user_form_data)

        self.assertTrue(auth.get_user(self.client).is_anonymous)

        response = self.client.post(reverse('login'),
                                    data={
                                        'username': self.user_data['username'],
                                        'password': self.user_data['password1']
                                    }, follow=True)

        self.assertTrue(auth.get_user(self.client).is_anonymous)
        self.assertContains(response, 'Your registration is currently pending administrator approval.')
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn(f'Registration Request from {self.user_data["first_name"]} {self.user_data["last_name"]}',
                      mail.outbox[0].subject)

    @patch('tom_registration.registration_flows.approval_required.views.mail_managers')
    def test_user_register_email_send_failure(self, mock_mail_managers):
        """Test that a registration email send failure logs the correct error message."""
        mock_mail_managers.side_effect = SMTPException('exception content')
        with self.assertLogs('tom_registration.registration_flows.approval_required.views', level='ERROR') as logs:
            self.client.post(reverse('registration:register'), data=self.user_form_data)
            self.assertIn(
                'ERROR:tom_registration.registration_flows.approval_required.views:'
                'Unable to send email: exception content',
                logs.output)

    def test_user_approve(self):
        """Test that a user can log in following approval in the approval registration flow."""
        self.client.post(reverse('registration:register'), data=self.user_form_data)
        user = User.objects.get(username=self.user_data['username'])
        self.assertFalse(user.is_active)

        self.client.force_login(self.superuser)
        self.client.post(reverse('registration:approve', kwargs={'pk': user.id}), data=self.user_form_data)
        user.refresh_from_db()
        self.assertTrue(user.is_active)
        self.assertEqual(len(mail.outbox), 2)  # There should be two--one for the registration, one for the approval
        self.assertIn(f'Your {settings.TOM_NAME} registration has been approved!', mail.outbox[1].subject)

    @patch('tom_registration.registration_flows.approval_required.views.send_mail')
    def test_user_approve_email_send_failure(self, mock_send_mail):
        """Test that an approval email send failure logs the correct error message."""
        self.client.force_login(self.superuser)
        test_user_data = copy.copy(self.user_data)
        test_user_data['password'] = test_user_data.pop('password1')
        test_user_data.pop('password2')
        user = User.objects.create(**test_user_data, is_active=False)
        mock_send_mail.side_effect = SMTPException('exception content')

        with self.assertLogs('tom_registration.registration_flows.approval_required.views', level='ERROR') as logs:
            self.client.post(reverse('registration:approve', kwargs={'pk': user.id}), data={**test_user_data,
                                                                                            **self.form_data})
            self.assertIn(
                'ERROR:tom_registration.registration_flows.approval_required.views:'
                'Unable to send email: exception content',
                logs.output)


@override_settings(MIDDLEWARE=[
                        'django.middleware.security.SecurityMiddleware',
                        'django.contrib.sessions.middleware.SessionMiddleware',
                        'django.middleware.common.CommonMiddleware',
                        'django.middleware.csrf.CsrfViewMiddleware',
                        'django.contrib.auth.middleware.AuthenticationMiddleware',
                        'django.contrib.messages.middleware.MessageMiddleware',
                        'django.middleware.clickjacking.XFrameOptionsMiddleware',
                        'tom_common.middleware.Raise403Middleware',
                        'tom_common.middleware.ExternalServiceMiddleware',
                        'tom_common.middleware.AuthStrategyMiddleware',
                        'tom_registration.middleware.RedirectAuthenticatedUsersFromRegisterMiddleware'])
class TestMiddleware(TestCase):
    """NOTE: run tests with: python ./tom_registration/tests/run_tests.py"""
    def setUp(self):
        self.user = User.objects.create(username='testuser')

    def test_redirect_authenticated_user_to_profile(self):
        """Test that an authenticated user is redirected to their profile if they attempt to reach the register page."""
        self.client.force_login(self.user)
        response = self.client.get(reverse('registration:register'), follow=True)
        self.assertContains(response, 'Update')

    def test_no_redirection_for_anonymous_user(self):
        """
        Test that an anonymous user can successfully access the register page with the
        RedirectAuthenticatedUsersFromRegisterMiddleware enabled.
        """
        response = self.client.get(reverse('registration:register'), follow=True)
        self.assertNotContains(response, 'Update')
        self.assertContains(response, 'Register')



from django.shortcuts import redirect
from django.urls import reverse


class RedirectAuthenticatedUsersFromRegisterMiddleware:
    """
    Middleware used to redirect authenticated users away from the register page.
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user.is_authenticated and request.path_info == reverse('registration:register'):
            return redirect(reverse('user-update', kwargs={'pk': request.user.id}))

        return self.get_response(request)



import logging

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import login
from django.contrib.auth.models import User, Group
from django.views.generic.edit import CreateView
from django.shortcuts import redirect
from django.urls import reverse_lazy

from tom_registration.registration_flows.open.forms import OpenRegistrationForm
from tom_registration import __version__

logger = logging.getLogger(__name__)


class OpenRegistrationView(CreateView):
    """
    View for registering in the open registration flow. This view creates the user, adds them to the public group, and
    immediately logs them in.
    """
    template_name = 'tom_registration/register_user.html'
    success_url = reverse_lazy(settings.TOM_REGISTRATION.get('REGISTRATION_REDIRECT_PATTERN', 'home'))
    form_class = OpenRegistrationForm
    extra_context = {'version': __version__}

    def form_valid(self, form):
        super().form_valid(form)
        group, _ = Group.objects.get_or_create(name='Public')
        group.user_set.add(self.object)
        group.save()

        messages.info(self.request, 'Registration was successful!')
        if isinstance(self.object, User):
            try:
                login(self.request, self.object,
                      backend=settings.TOM_REGISTRATION.get('REGISTRATION_AUTHENTICATION_BACKEND'))
            except ValueError as ve:
                logger.error(f'Unable to log in newly registered user: {ve}')

        return redirect(self.get_success_url())



from django import template
from django.contrib.auth.models import User

register = template.Library()


@register.inclusion_tag('tom_registration/partials/register_button.html')
def registration_button():
    """
    Renders a register button that navigates the user to the registration form.
    """
    return


@register.inclusion_tag('tom_registration/partials/pending_users.html', takes_context=True)
def pending_users(context: dict) -> dict:
    """
    Renders a table of pending (AKA inactive) users with buttons to approve or delete the user.
    """
    return {
        'request': context['request'],
        'users': User.objects.filter(is_active=False)
    }



# boot_django.py
#
# This file sets up and configures Django. It's used by scripts that need to
# execute as if running in a Django server.

import os
import django
from django.conf import settings

APP_NAME = 'tom_registration'  # the stand-alone app we are testing

BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), APP_NAME))


def boot_django():
    settings.configure(
        BASE_DIR=BASE_DIR,
        # SECURITY WARNING: keep the secret key used in production secret!
        SECRET_KEY='v5j-rg7sc+leg-m+vf947vi34+fs1%+$m%*l%sb7^fnwb$-29y',
        DEBUG=True,
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
            }
        },
        TOM_NAME='Test TOM',
        INSTALLED_APPS=(
            'django.contrib.admin',
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.messages',
            'django.contrib.staticfiles',
            'django.contrib.sites',
            'django_extensions',
            'guardian',
            'tom_common',
            'django_comments',
            'bootstrap4',
            'crispy_forms',
            'rest_framework',
            'rest_framework.authtoken',
            'django_filters',
            'django_gravatar',
            'tom_targets',
            'tom_alerts',
            'tom_catalogs',
            'tom_observations',
            'tom_dataproducts',
            APP_NAME,  # defined above
        ),
        SITE_ID=1,
        EXTRA_FIELDS={},
        TIME_ZONE='UTC',
        USE_TZ=True,
        HOOKS={
            'target_post_save': 'tom_common.hooks.target_post_save',
            'observation_change_state': 'tom_common.hooks.observation_change_state',
            'data_product_post_upload': 'tom_dataproducts.hooks.data_product_post_upload'
        },
        MIDDLEWARE=[
            'django.middleware.security.SecurityMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
            'tom_common.middleware.Raise403Middleware',
            'tom_common.middleware.ExternalServiceMiddleware',
            'tom_common.middleware.AuthStrategyMiddleware',
        ],
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [os.path.join(BASE_DIR, 'templates')],
                'APP_DIRS': True,
                'OPTIONS': {
                    'context_processors': [
                        'django.template.context_processors.debug',
                        'django.template.context_processors.request',
                        'django.contrib.auth.context_processors.auth',
                        'django.contrib.messages.context_processors.messages',
                    ],
                },
            },
        ],
        AUTHENTICATION_BACKENDS=(
            'django.contrib.auth.backends.ModelBackend',
            'guardian.backends.ObjectPermissionBackend',
        ),
        AUTH_STRATEGY='READ_ONLY',
        STATIC_URL='/static/',
        STATIC_ROOT=os.path.join(BASE_DIR, '_static'),
        STATICFILES_DIRS=[os.path.join(BASE_DIR, 'static')],
        MEDIA_ROOT=os.path.join(BASE_DIR, 'data'),
        MEDIA_URL='/data/',
        ROOT_URLCONF='tom_registration.tests.urls.test_urls',
        TOM_REGISTRATION={
            'REGISTRATION_AUTHENTICATION_BACKEND': 'django.contrib.auth.backends.ModelBackend',
            'REGISTRATION_REDIRECT_PATTERN': 'home',
            'SEND_APPROVAL_EMAILS': True,
            'REGISTRATION_STRATEGY': 'open'  # ['open', 'approval_required']
        },
        FACILITIES={
            'LCO': {
                'portal_url': 'https://observe.lco.global',
                'api_key': '',
            },
            'GEM': {
                'portal_url': {
                    'GS': 'https://139.229.34.15:8443',
                    'GN': 'https://128.171.88.221:8443',
                },
                'api_key': {
                    'GS': '',
                    'GN': '',
                },
                'user_email': '',
                'programs': {
                    'GS-YYYYS-T-NNN': {
                        'MM': 'Std: Some descriptive text',
                        'NN': 'Rap: Some descriptive text'
                    },
                    'GN-YYYYS-T-NNN': {
                        'QQ': 'Std: Some descriptive text',
                        'PP': 'Rap: Some descriptive text',
                    },
                },
            },
        },
        TOM_FACILITY_CLASSES=[
            'tom_observations.facilities.lco.LCOFacility',
            'tom_observations.facilities.gemini.GEMFacility',
            'tom_observations.facilities.soar.SOARFacility',
            'tom_observations.facilities.lt.LTFacility'
        ]
    )
    django.setup()



from django.contrib.auth.views import LoginView
from django.urls import path

from tom_registration.registration_flows.open.views import OpenRegistrationView

app_name = 'tom_registration'


urlpatterns = [
    path('accounts/login/', LoginView.as_view(), name='login'),
    path('accounts/register/', OpenRegistrationView.as_view(), name='register'),
]



import logging
from smtplib import SMTPException

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import User, Group
from django.contrib.sites.models import Site
from django.core.mail import mail_managers, send_mail
from django.views.generic.edit import CreateView, UpdateView
from django.shortcuts import redirect, reverse
from django.urls import reverse_lazy

from tom_common.mixins import SuperuserRequiredMixin
from tom_registration.registration_flows.approval_required.forms import ApproveUserForm
from tom_registration.registration_flows.approval_required.forms import RegistrationApprovalForm
from tom_registration import __version__

logger = logging.getLogger(__name__)


# TODO: Add post-approval hooks that actually handle the sending of email
class ApprovalRegistrationView(CreateView):
    """
    View for handling registration requests in the approval required registration flow. This flow creates users but sets
    them as inactive, requiring administrator approval in order to log in. Upon registration, an email is sent to the
    administrators of the TOM informing them of the request.
    """
    template_name = 'tom_registration/register_user.html'
    success_url = reverse_lazy(settings.TOM_REGISTRATION.get('REGISTRATION_REDIRECT_PATTERN', ''))
    form_class = RegistrationApprovalForm
    extra_context = {'version': __version__}

    def form_valid(self, form):
        super().form_valid(form)
        group, _ = Group.objects.get_or_create(name='Public')
        group.user_set.add(self.object)
        group.save()

        messages.info(self.request, 'Your request to register has been submitted to the administrators.')

        if settings.TOM_REGISTRATION.get('SEND_APPROVAL_EMAILS'):
            try:
                current_domain = Site.objects.get_current().domain
                link_to_user_list = f'https://{current_domain}{reverse("user-list")}'
                mail_managers(
                    subject=f'Registration Request from {self.object.first_name} {self.object.last_name}',
                    message='',  # leave this blank in favor of html_message
                    fail_silently=False,
                    html_message=f'{self.object.first_name} {self.object.last_name} '
                                 f'has requested to register in your TOM. Please approve or delete this user'
                                 f' <a href="{link_to_user_list}">here</a>.',
                )
            except (SMTPException, ConnectionRefusedError) as error:
                logger.error(f'Unable to send email: {error}')

        return redirect(self.get_success_url())


class UserApprovalView(SuperuserRequiredMixin, UpdateView):
    """
    View for approving (activating) pending (inactive) users in the approval required registration flow. Upon approval,
    an email is sent to the user informing them of the registration approval.
    """
    model = User
    template_name = 'tom_registration/approve_user.html'
    success_url = reverse_lazy('user-list')
    form_class = ApproveUserForm

    def form_valid(self, form):
        response = super().form_valid(form)

        if settings.TOM_REGISTRATION.get('SEND_APPROVAL_EMAILS'):
            try:
                current_domain = Site.objects.get_current().domain
                link_to_login = f'https://{current_domain}{reverse("login")}'
                send_mail(
                    subject=settings.TOM_REGISTRATION.get('APPROVAL_SUBJECT',
                                                          f'Your {settings.TOM_NAME} '
                                                          f'registration has been approved!'),
                    message='',  # leave this blank in favor of html_message
                    from_email=settings.SERVER_EMAIL,
                    recipient_list=[self.object.email],
                    html_message=settings.TOM_REGISTRATION.get(
                        'APPROVAL_MESSAGE',
                        f'Your {settings.TOM_NAME} registration has been approved. '
                        f'You can log in <a href="{link_to_login}">here</a>.'
                    ),
                    fail_silently=False)
            except (SMTPException, ConnectionRefusedError) as error:
                logger.error(f'Unable to send email: {error}')

        return response



from django import forms
from django.contrib.auth.forms import AuthenticationForm

from tom_common.forms import CustomUserCreationForm


class RegistrationApprovalForm(CustomUserCreationForm):
    """
    Form for handling registration requests in the approval required registration flow. Sets the user to inactive.
    """
    def save(self, commit=True):
        user = super().save(commit=False)
        user.is_active = False
        if self.cleaned_data['password1']:
            user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
            self.save_m2m()

        return user


class ApproveUserForm(CustomUserCreationForm):
    """
    Form for handling user registration approval requests.
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields.pop('password1')
        self.fields.pop('password2')

    def save(self, commit=True):
        # NOTE: The superclass call is specifically to forms.ModelForm rather than CustomUserCreationForm--
        # this is done because the form doubles as an update form, and it bypasses any password checks.
        user = super(forms.ModelForm, self).save(commit=False)
        user.is_active = True
        if commit:
            user.save()
            self.save_m2m()

        return user


class ApprovalAuthenticationForm(AuthenticationForm):
    """
    Form that replaces the default Django AuthenticationForm and renders an appropriate message if an inactive user
    attempts to log in.
    """
    def confirm_login_allowed(self, user):
        if not user.is_active:
            raise forms.ValidationError('Your registration is currently pending administrator approval.',
                                        code='inactive'
                                        )



from tom_common.forms import CustomUserCreationForm


class OpenRegistrationForm(CustomUserCreationForm):
    """
    Form for handling open registration requests. Does not render any groups, as the user cannot choose their own
    groups.
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields.pop('groups')



#!/usr/bin/env python
# django_shell.py

import argparse

from django.core.management import call_command
from boot_django import boot_django, APP_NAME  # noqa


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('testname', default='', nargs='?',
                        help='Module paths to test; '
                             'can be modulename, modulename.TestCase or modulename.TestCase.test_method')
    parser.add_argument('-v', '--verbosity', default=2, type=int, choices=[0, 1, 2, 3],
                        help='Verbosity level;'
                             ' 0=minimal output, 1=normal output, 2=verbose output, 3=very verbose output')
    parser.add_argument('--canary', action='store_true', help='Run only canary tests (skips canary tests by default)')

    testname = parser.parse_args().testname
    verbosity = parser.parse_args().verbosity
    run_canary = parser.parse_args().canary

    # We still only want to test this app's tests, so if no specific test is supplied,
    # we use the app name to restrict testing
    if not testname:
        testname = APP_NAME

    # If the canary flag is set, we want to run only the canary tests, otherwise we want to exclude them
    if run_canary:
        canary_command_flag = '--tag=canary'
    else:
        canary_command_flag = '--exclude-tag=canary'

    boot_django()
    print(f'running test(s) for {APP_NAME}')
    call_command('test', testname, canary_command_flag, verbosity=verbosity)



from django.contrib.auth.views import LoginView
from django.urls import path

from tom_registration.registration_flows.approval_required.forms import ApprovalAuthenticationForm
from tom_registration.registration_flows.approval_required.views import ApprovalRegistrationView, UserApprovalView

app_name = 'tom_registration'


urlpatterns = [
    path('accounts/login/', LoginView.as_view(authentication_form=ApprovalAuthenticationForm), name='login'),
    path('accounts/register/', ApprovalRegistrationView.as_view(), name='register'),
    path('accounts/approve/<int:pk>/', UserApprovalView.as_view(), name='approve')
]



from django.apps import AppConfig
from django.urls import path, include
from django.conf import settings


class TomRegistrationConfig(AppConfig):
    name = 'tom_registration'

    def include_url_paths(self):
        """
        Integration point for adding URL patterns to the Tom Common URL configuration.
        This method should return a list of URL patterns to be included in the main URL configuration.
        """
        registration_strategy = settings.TOM_REGISTRATION['REGISTRATION_STRATEGY']
        urlpatterns = [
            path('', include(f'tom_registration.registration_flows.{registration_strategy}.urls',
                             namespace='registration')),
        ]
        return urlpatterns

    def nav_items(self):
        """
        Integration point for adding items to the navbar.
        This method should return a list of partial templates to be included in the navbar.
        """
        return [{'partial': 'tom_registration/partials/register_button.html', 'position': 'right'}]

    def user_lists(self):
        """
        Integration point for adding items to the user list page.

        This method should return a list of dictionaries that include a `partial` key pointing to the path of the html
        user_list partial. The `context` key should point to the dot separated string path to the templatetag that will
        return a dictionary containing new context for the accompanying partial.
        Typically, this partial will be a bootstrap table displaying some app specific user list or similar.

        """
        if settings.TOM_REGISTRATION['REGISTRATION_STRATEGY'] == 'approval_required':
            return [{'partial': 'tom_registration/partials/pending_users.html',
                     'context': 'tom_registration.templatetags.registration_extras.pending_users'}]
        else:
            return []
